import unittest
from lang.builtins import *


class TestDirections(unittest.TestCase):

    def test_min_dir(self):
        self.assertEqual(minDir(), Norte)

    def test_max_dir(self):
        self.assertEqual(maxDir(), Oeste)

    def test_next_dir(self):
        dirs = [Oeste, Norte, Este, Sur, Oeste]
        for i in range(len(dirs) - 1):
            self.assertEqual(siguiente(dirs[i]), dirs[i+1])

    def test_prev_dir(self):
        dirs = [Oeste, Norte, Este, Sur, Oeste]
        for i in range(1, len(dirs)):
            self.assertEqual(previo(dirs[i]), dirs[i-1])

    def test_opposite_dir(self):
        dirs = [Norte, Este, Sur, Oeste]
        opposites = [Sur, Oeste, Norte, Este]
        for i in range(len(dirs)):
            self.assertEqual(opuesto(dirs[i]), opposites[i])


if __name__ == '__main__':
    unittest.main()



import sys
from ..i18n import translate as t
from .types import *
from ..state import state as global_state
from ..exceptions import DynamicException

# Board Procedures

def PutStone(color):
    """Put a stone in the board."""
    if typeof(color) != 'Color':
        msg = t('The argument to PutStone should be a color')
        raise RuntimeException(msg)
    global_state.board.put_stone(color)

def TakeStone(color):
    """Take a stone from the board."""
    if typeof(color) != 'Color':
        msg = t('The argument to TakeStone should be a color')
        raise RuntimeException(msg)
    if global_state.board.num_stones(color) > 0:
        global_state.board.take_stone(color)
    else:
        msg = global_state.backtrace(
            t('Cannot take stones of color %s') % (color,))
        raise RuntimeException(msg)

def Move(direction):
    """Move the head."""
    if typeof(direction) != 'Dir':
        msg = t('The argument to Move should be a direction')
        raise RuntimeException(msg)
    if global_state.board.can_move(direction):
        global_state.board.move(direction)
    else:
        msg = global_state.backtrace(
            t('Cannot move to %s') % (direction,))
        raise RuntimeException(msg)

def GoToOrigin():
    return global_state.board.go_to_origin()

def GoToBoundary(direction):
    return global_state.board.go_to_boundary(direction)

def ClearBoard():
    return global_state.board.clear_board()

# Board Functions

def numStones(color):
    """Number of stones of the given color."""
    if typeof(color) != 'Color':
        msg = t('The argument to numStones should be a color')
        raise RuntimeException(msg)
    return global_state.board.num_stones(color)

def existStones(color):
    """Return True iff there are stones of the given color."""
    if typeof(color) != 'Color':
        msg = t('The argument to existStones should be a color')
        raise RuntimeException(msg)
    return global_state.board.exist_stones(color)

def canMove(direction):
    """Return True iff the head can move to the given direction."""
    if typeof(direction) != 'Dir':
        msg = t('The argument to canMove should be a direction')
        raise RuntimeException(msg)
    return global_state.board.can_move(direction)



# General Procedures

def Imprimir(*values):
    strings = list(map(str, values))
    print('"' + ' '.join(strings) + '"')


# General Functions

minBool  = lambda: False
maxBool  = lambda: True
minDir   = lambda: DIRECTIONS[0]
maxDir   = lambda: DIRECTIONS[-1]
minColor = lambda: COLORS[0]
maxColor = lambda: COLORS[-1]

def next(value):
    "Return the next value of the same type as the one given."
    if isinstance(value, bool):
        return not value
    elif isinteger(value):
        return value + 1
    elif isinstance(value, list):
        return [next(elem) for elem in value]
    elif isenum(value):
        return value.next()
    else:
        assert False

def prev(value):
    "Return the previous value of the same type as the one given."
    if isinstance(value, bool):
        return not value
    elif isinteger(value):
        return value - 1
    elif isinstance(value, list):
        return [prev(elem) for elem in value]
    elif isenum(value):
        return value.prev()
    else:
        assert False

def opposite(value):
    "Return the opposite value of the same type as the one given."
    if isinstance(value, bool):
        return not value
    elif isinteger(value):
        return -value
    elif isinstance(value, list):
        return [opposite(elem) for elem in value]
    elif isenum(value):
        return value.opposite()
    else:
        assert False

def ord(value):
    """Returns an integer representing the ord of the given
    Gobstones value."""
    if isinstance(value, bool):
        if value:
            return 1
        else:
            return 0
    elif isinteger(value):
        return value
    elif isenum(value):
        return value.ord()
    else:
        assert False

# Private

class RuntimeException(DynamicException):
    """Base exception for Gobstones runtime errors."""

    def error_type(self):
        "Description of the exception type."
        return t('Runtime error')

def isinteger(value):
    "Return True iff the given Python value is integral."
    if sys.version_info[0] < 3:
        return isinstance(value, int) or isinstance(value, long)
    else:
        return isinstance(value, int)

def isenum(value):
    return hasattr(value, 'ord') and hasattr(value.ord, '__call__')

def typeof(value):
    "Return the name of the type of the value."
    if isinstance(value, bool):
        return 'Bool'
    elif isinteger(value):
        return 'Int'
    elif isinstance(value, list):
        assert False
    else:
        return value.enum_type()


def define_function(_def):

    def function(*args, **kwargs):
        global_state.push()
        result = _def(*args, **kwargs)
        global_state.pop()
        return result

    return function

# Saving locals
# This line must be the last line on this file.
builtins = locals()



#
# Copyright (C) 2011-2015 Pablo Barenbaum <foones@gmail.com>,
#                         Ary Pablo Batista <arypbatista@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

import re
import json

from ..utils import *
from ..i18n import translate as t
from ..builtins.types.color import NUM_COLORS, COLORS_BY_NAME, COLORS, COLOR_NAMES, Color
from .basic import BoardFormatException, BoardFormat

def is_numeric(x):
  return re.match('^[0-9]+$', x)

class JsonBoardFormat(BoardFormat):
  "Simple human-friendly board format."

  def dump(self, board, f=None, style='verbose', **kwargs):
    if style == 'verbose':
      output = self.dump_with_translator(board, self.verbose_translator)
    elif style == 'compact':
      output = self.dump_with_translator(board, self.compact_translator)
    else:
      assert False

    if not f is None:
        f.write(output)
    return json.dumps(output)

  def dump_to_dict(self, board, style='verbose', **kwargs):
    if style == 'verbose':
      output = self.dump_with_translator(board, self.verbose_translator)
    elif style == 'compact':
      output = self.dump_with_translator(board, self.compact_translator)
    else:
      assert False
    return output

  def verbose_translator(self, s):
    return s

  def compact_translator(self, s):
    _dict = {
      'cell': 'o',
      'size': 's',
      'head': 'x',
    }
    if s in _dict:
      return _dict[s]
    else:
      return s[0]

  def dump_with_translator(self, board, translate):
    w, h = board.size
    output = {"size" : board.size}
    cells = {}
    for x in range(w):
      for y in range(h):
        cell = {}
        for coli in range(4):
          cant = board.cells[y][x].num_stones(coli)
          if cant == 0: continue
          col = Color(coli).i18n_name()
          cell.update({col : cant})
        if cell == {}: continue

        if not x in cells.keys():
            cells[x] = {}
        cells[x][y] = cell
    output.update({"head": board.head})
    output.update({"cells": cells})
    return output

  def load(self, board, f):
      raise Exception("Not implemented")



import lang
from lang.utils import read_file
import lang.i18n as i18n
import argparse

def parse_args():
    parser = argparse.ArgumentParser(description='Execute Gobspy.')
    parser.add_argument('file')
    parser.add_argument('-l', '--language', default='es',
        help='language used for code translation')

    args = parser.parse_args()
    return args



def main():
    args = parse_args()
    i18n.set_language(args.language
    )
    lang.run(read_file(args.file), args.file)
    print(lang.state.board)

if __name__ == '__main__':
    main()



#
# Copyright (C) 2011-2017 Ary Pablo Batista <arypbatista@gmail.com>, Pablo Barenbaum <foones@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
from ..utils import *
from ..exceptions import BoardFormatException

class BoardFormat(object):
    def to_string(self, board, **kwargs):
        out = StringIO()
        self.dump(board, out, **kwargs)
        res = out.getvalue()
        out.close()
        return res
    def from_string(self, s, board=None):
        from . import Board
        if board is None:
            board = Board()
        f = StringIO(s)
        self.load(board, f)
        f.close()
        return board



from .enum import NamedEnum
from ...i18n import translate as t

#### Colors

COLOR_NAMES = [
    'Color0',
    'Color1',
    'Color2',
    'Color3',
]

class Color(NamedEnum):
    "Represents a Gobstones color."

    def enum_type(self):
        "Return the name of the enumerated type."
        return 'Color'

    def enum_size(self):
        "Return the size of the enumerated type."
        return 4

    def name(self):
        "Return the name of this color."
        return COLOR_NAMES[self.ord()]

    def __repr__(self):
        return self.name()


COLORS = [Color(i) for i in range(len(COLOR_NAMES))]

COLORS_BY_NAME = dict([(c.name(), c) for c in COLORS])

I18N_COLOR_NAMES = [c.i18n_name() for c in COLORS]
I18N_COLORS_BY_NAME = dict([(c.i18n_name(), c) for c in COLORS])

NUM_COLORS = len(COLORS)
